=== modified file 'lib/canonical/launchpad/security.py'
--- lib/canonical/launchpad/security.py	2011-07-11 03:49:12 +0000
+++ lib/canonical/launchpad/security.py	2011-07-13 10:35:30 +0000
@@ -175,7 +175,7 @@
     IBinaryPackageReleaseDownloadCount,
     )
 from lp.soyuz.interfaces.buildfarmbuildjob import IBuildFarmBuildJob
-from lp.soyuz.interfaces.packagecopyjob import IPackageCopyJobEdit
+from lp.soyuz.interfaces.packagecopyjob import IPlainPackageCopyJob
 from lp.soyuz.interfaces.packageset import (
     IPackageset,
     IPackagesetSet,
@@ -1450,14 +1450,18 @@
         return not permissions.is_empty()
 
 
-class EditPackageCopyJob(AuthorizationBase):
+class EditPlainPackageCopyJob(AuthorizationBase):
     permission = 'launchpad.Edit'
-    usedfor = IPackageCopyJobEdit
+    usedfor = IPlainPackageCopyJob
 
     def checkAuthenticated(self, user):
+        archive = self.obj.target_archive
+        if archive.is_ppa:
+            return archive.checkArchivePermission(user.person)
+
         permission_set = getUtility(IArchivePermissionSet)
         permissions = permission_set.componentsForQueueAdmin(
-            self.obj.target_archive, user.person)
+            archive, user.person)
         return not permissions.is_empty()
 
 

=== modified file 'lib/lp/soyuz/configure.zcml'
--- lib/lp/soyuz/configure.zcml	2011-06-25 13:53:41 +0000
+++ lib/lp/soyuz/configure.zcml	2011-07-12 14:23:40 +0000
@@ -928,15 +928,21 @@
 
     <!-- PackageCopyJob -->
     <class class=".model.packagecopyjob.PackageCopyJob">
-      <allow interface=".interfaces.packagecopyjob.IPackageCopyJobEdit"/>
-      <allow interface=".interfaces.packagecopyjob.IPackageCopyJob"/>
-      <require permission="launchpad.Edit" set_schema=".interfaces.packagecopyjob.IPackageCopyJobEdit"/>
+      <allow interface=".interfaces.packagecopyjob.IPackageCopyJobPublic"/>
+      <require
+          permission="launchpad.Edit"
+          set_schema=".interfaces.packagecopyjob.IPackageCopyJobPublic"
+          interface=".interfaces.packagecopyjob.IPackageCopyJobEdit"/>
     </class>
 
     <!-- PlainPackageCopyJob -->
     <class class=".model.packagecopyjob.PlainPackageCopyJob">
-      <allow interface=".interfaces.packagecopyjob.IPackageCopyJob" />
+      <allow interface=".interfaces.packagecopyjob.IPackageCopyJobPublic" />
       <allow interface=".interfaces.packagecopyjob.IPlainPackageCopyJob" />
+      <require
+          permission="launchpad.Edit"
+          set_schema=".interfaces.packagecopyjob.IPackageCopyJobPublic"
+          interface=".interfaces.packagecopyjob.IPackageCopyJobEdit"/>
     </class>
 
     <!-- PackageCopyPolicy -->

=== modified file 'lib/lp/soyuz/interfaces/packagecopyjob.py'
--- lib/lp/soyuz/interfaces/packagecopyjob.py	2011-06-24 18:18:47 +0000
+++ lib/lp/soyuz/interfaces/packagecopyjob.py	2011-07-12 14:23:40 +0000
@@ -61,7 +61,7 @@
         """Update the job's JSON metadata with items from `metadata_dict`."""
 
 
-class IPackageCopyJob(Interface):
+class IPackageCopyJobPublic(Interface):
     """The immutable data on an `IPackageCopyJob`, for normal use."""
 
     id = Int(
@@ -107,6 +107,10 @@
     metadata = Attribute(_("A dict of data about the job."))
 
 
+class IPackageCopyJob(IPackageCopyJobPublic, IPackageCopyJobEdit):
+    """An `IJob` representing a copy of packages between places."""
+
+
 class PackageCopyJobType(DBEnumeratedType):
 
     PLAIN = DBItem(1, """

=== modified file 'lib/lp/soyuz/tests/test_packagecopyjob.py'
--- lib/lp/soyuz/tests/test_packagecopyjob.py	2011-07-07 10:31:38 +0000
+++ lib/lp/soyuz/tests/test_packagecopyjob.py	2011-07-12 14:52:52 +0000
@@ -10,7 +10,9 @@
     )
 import transaction
 from zope.component import getUtility
-from zope.security.interfaces import ForbiddenAttribute
+from zope.security.interfaces import (
+    Unauthorized,
+    )
 from zope.security.proxy import removeSecurityProxy
 
 from storm.store import Store
@@ -18,7 +20,8 @@
 from canonical.config import config
 from canonical.launchpad.interfaces.lpstorm import IStore
 from canonical.launchpad.webapp.testing import verifyObject
-from canonical.testing import (
+from canonical.testing import (\
+    LaunchpadFunctionalLayer,
     LaunchpadZopelessLayer,
     ZopelessDatabaseLayer,
     )
@@ -63,6 +66,7 @@
     run_script,
     TestCaseWithFactory,
     )
+from lp.testing import person_logged_in
 from lp.testing.mail_helpers import pop_notifications
 from lp.testing.matchers import Provides
 from lp.testing.fakemethod import FakeMethod
@@ -862,10 +866,44 @@
             self.assertEqual(
                 policy, naked_job.getPolicyImplementation().enum_value)
 
-    def test_extendMetadata_is_privileged(self):
+
+class TestPlainPackageCopyJobPermissions(TestCaseWithFactory):
+
+    layer = LaunchpadFunctionalLayer
+
+    def test_extendMetadata_edit_privilege_by_queue_admin(self):
+        # A person who has any queue admin rights can edit the copy job.
+        archive = self.factory.makeArchive(purpose=ArchivePurpose.PRIMARY)
+        pcj = self.factory.makePlainPackageCopyJob(target_archive=archive)
+        queue_admin = self.factory.makePerson()
+        with person_logged_in(pcj.target_archive.owner):
+            pcj.target_archive.newQueueAdmin(queue_admin, "main")
+        with person_logged_in(queue_admin):
+            # This won't blow up.
+            pcj.extendMetadata({})
+
+    def test_extendMetadata_edit_privilege_by_other(self):
+        # Random people cannot edit the copy job.
         pcj = self.factory.makePlainPackageCopyJob()
         self.assertRaises(
-            ForbiddenAttribute, getattr, pcj, "extendMetadata")
+            Unauthorized, getattr, pcj, "extendMetadata")
+
+    def test_PPA_edit_privilege_by_owner(self):
+        # A PCJ for a PPA allows the PPA owner to edit it.
+        ppa = self.factory.makeArchive(purpose=ArchivePurpose.PPA)
+        pcj = self.factory.makePlainPackageCopyJob(target_archive=ppa)
+        with person_logged_in(ppa.owner):
+            # This will not throw an exception.
+            pcj.extendMetadata({})
+
+    def test_PPA_edit_privilege_by_other(self):
+        # A PCJ for a PPA does not allow non-owners to edit it.
+        ppa = self.factory.makeArchive(purpose=ArchivePurpose.PPA)
+        pcj = self.factory.makePlainPackageCopyJob(target_archive=ppa)
+        person = self.factory.makePerson()
+        with person_logged_in(person):
+            self.assertRaises(
+                Unauthorized, getattr, pcj, "extendMetadata")
 
 
 class TestPlainPackageCopyJobDbPrivileges(TestCaseWithFactory,

